@@ -17,21 +17,8 @@ buffer* buffer_init(void) {
 	return b;
 }
 
-buffer *buffer_init_buffer(const buffer *src) {
-	buffer * const b = buffer_init();
-	buffer_copy_string_len(b, BUF_PTR_LEN(src));
-	return b;
-}
-
-buffer *buffer_init_string(const char *str) {
-	buffer * const b = buffer_init();
-	buffer_copy_string(b, str);
-	return b;
-}
-
 void buffer_free(buffer *b) {
 	if (NULL == b) return;
-
 	free(b->ptr);
 	free(b);
 }
@@ -166,8 +153,10 @@ void buffer_commit(buffer *b, size_t size)
 	b->ptr[sz - 1] = '\0';
 }
 
+__attribute_cold__ /*(reduce code size due to inlining)*/
 void buffer_copy_string(buffer * restrict b, const char * restrict s) {
-	buffer_copy_string_len(b, s, NULL != s ? strlen(s) : 0);
+    if (__builtin_expect( (NULL == s), 0)) s = "";
+    buffer_copy_string_len(b, s, strlen(s));
 }
 
 void buffer_copy_string_len(buffer * const restrict b, const char * const restrict s, const size_t len) {
@@ -179,8 +168,10 @@ void buffer_copy_string_len(buffer * const restrict b, const char * const restri
     memcpy(d, s, len);
 }
 
+__attribute_cold__ /*(reduce code size due to inlining)*/
 void buffer_append_string(buffer * restrict b, const char * restrict s) {
-	buffer_append_string_len(b, s, NULL != s ? strlen(s) : 0);
+    if (__builtin_expect( (NULL == s), 0)) s = "";
+    buffer_append_string_len(b, s, strlen(s));
 }
 
 /**
@@ -667,93 +658,15 @@ void buffer_append_string_encoded(buffer * const restrict b, const char * const
 	}
 }
 
-void buffer_append_string_encoded_json(buffer * const restrict b, const char * const restrict s, const size_t len) {
-	const unsigned char * const restrict ds = (unsigned char *)s;
-	size_t dlen = 0;
-
-	/* calculate space needed for string including encodings */
-	for (size_t i = 0; i < len; ++i) {
-		int c = ds[i];
-		if (c == '"' || c == '\\' || c < 0x20 || c == 0x7f) {
-			switch (c) {
-			  case '\b':
-			  case '\t':
-			  case '\n':
-			  case '\f':
-			  case '\r':
-			  case '"':
-			  case '\\':
-				dlen += 2;
-				break;
-			  default:
-				dlen += 6; /* \uCCCC */
-				break;
-			}
-		}
-		else {
-			++dlen;
-		}
-	}
-
-	unsigned char * const d = (unsigned char *)buffer_extend(b, dlen);
-
-	if (__builtin_expect( (dlen == len), 1)) {/*(short-circuit; nothing to encode)*/
-		memcpy(d, ds, len);
-		return;
-	}
-
-	dlen = 0;
-	for (size_t i = 0; i < len; ++i) {
-		int c = ds[i];
-		if (c == '"' || c == '\\' || c < 0x20 || c == 0x7f) {
-			d[dlen++] = '\\';
-			switch (c) {
-			  case '\b':
-				d[dlen++] = 'b';
-				break;
-			  case '\t':
-				d[dlen++] = 't';
-				break;
-			  case '\n':
-				d[dlen++] = 'n';
-				break;
-			  case '\f':
-				d[dlen++] = 'f';
-				break;
-			  case '\r':
-				d[dlen++] = 'r';
-				break;
-			  case '"':
-				d[dlen++] = '"';
-				break;
-			  case '\\':
-				d[dlen++] = '\\';
-				break;
-			  default:
-				d[dlen  ] = 'u';
-				d[dlen+1] = '0';
-				d[dlen+2] = '0';
-				d[dlen+3] = hex_chars_lc[(c >> 4) & 0x0F];
-				d[dlen+4] = hex_chars_lc[c & 0x0F];
-				dlen += 5;
-				break;
-			}
-		}
-		else {
-			d[dlen++] = c;
-		}
-	}
-}
-
-
 void buffer_append_string_c_escaped(buffer * const restrict b, const char * const restrict s, size_t s_len) {
 	unsigned char *ds, *d;
 	size_t d_len, ndx;
 
 	/* count to-be-encoded-characters */
 	for (ds = (unsigned char *)s, d_len = 0, ndx = 0; ndx < s_len; ds++, ndx++) {
-		if ((*ds < 0x20) /* control character */
-				|| (*ds >= 0x7f)) { /* DEL + non-ASCII characters */
+		if (__builtin_expect( (*ds >= ' ' && *ds <= '~'), 1))
+			d_len++;
+		else { /* CTLs or non-ASCII characters */
 			switch (*ds) {
 			case '\t':
 			case '\r':
@@ -764,8 +677,6 @@ void buffer_append_string_c_escaped(buffer * const restrict b, const char * cons
 				d_len += 4; /* \xCC */
 				break;
 			}
-		} else {
-			d_len++;
 		}
 	}
 
@@ -777,8 +688,9 @@ void buffer_append_string_c_escaped(buffer * const restrict b, const char * cons
 	}
 
 	for (ds = (unsigned char *)s, d_len = 0, ndx = 0; ndx < s_len; ds++, ndx++) {
-		if ((*ds < 0x20) /* control character */
-				|| (*ds >= 0x7f)) { /* DEL + non-ASCII characters */
+		if (__builtin_expect( (*ds >= ' ' && *ds <= '~'), 1))
+			d[d_len++] = *ds;
+		else { /* CTLs or non-ASCII characters */
 			d[d_len++] = '\\';
 			switch (*ds) {
 			case '\t':
@@ -792,17 +704,81 @@ void buffer_append_string_c_escaped(buffer * const restrict b, const char * cons
 				break;
 			default:
 				d[d_len++] = 'x';
-				d[d_len++] = hex_chars_lc[((*ds) >> 4) & 0x0F];
+				d[d_len++] = hex_chars_lc[(*ds) >> 4];
 				d[d_len++] = hex_chars_lc[(*ds) & 0x0F];
 				break;
 			}
-		} else {
-			d[d_len++] = *ds;
 		}
 	}
 }
 
 
+void
+buffer_append_bs_escaped (buffer * const restrict b,
+                          const char * restrict s, const size_t len,
+                          const buffer_bs_escape_t esc)
+{
+    /* replaces non-printable chars with escaped string
+     * default: \xHH where HH is the hex representation of the byte
+     * json: \u00HH where HH is the hex representation of the byte
+     * exceptions: " => \", \ => \\, whitespace chars => \n \t etc. */
+    /* Intended for use escaping string to be surrounded by double-quotes */
+    /* Performs single pass over string and is optimized for ASCII;
+     * non-ASCII escaping might be slightly sped up by walking input twice,
+     * first to calculate escaped length and extend the destination b, and
+     * second to do the escaping. (This non-ASCII optim is not done here) */
+    buffer_string_prepare_append(b, len);
+    for (const char * const end = s+len; s < end; ++s) {
+        unsigned int c;
+        const char * const ptr = s;
+        do {
+            c = *(const unsigned char *)s;
+        } while (c >= ' ' && c <= '~' && c != '"' && c != '\\' && ++s < end);
+        if (s - ptr) buffer_append_string_len(b, ptr, s - ptr);
+
+        if (s == end)
+            return;
+
+        /* ('\a', '\v' shortcuts are technically not json-escaping) */
+        /* ('\0' is also omitted due to the possibility of string corruption if
+         *  the receiver supports decoding octal escapes (\000) and the escaped
+         *  string contains \0 followed by two digits not part of escaping)*/
+
+        char *d;
+        switch (c) {
+          case '\a':case '\b':case '\t':case '\n':case '\v':case '\f':case '\r':
+            c = "0000000abtnvfr"[c];
+            __attribute_fallthrough__
+          case '"': case '\\':
+            d = buffer_extend(b, 2);
+            d[0] = '\\';
+            d[1] = c;
+            break;
+          default:
+            if (0 == esc) { /* BS_ESCAPE_DEFAULT */
+                /* non printable char => \xHH */
+                d = buffer_extend(b, 4);
+                d[0] = '\\';
+                d[1] = 'x';
+                d += 2;
+            }
+            else {          /* BS_ESCAPE_JSON */
+                /*(technically do not have to escape DEL (\127) or higher)*/
+                d = buffer_extend(b, 6);
+                d[0] = '\\';
+                d[1] = 'u';
+                d[2] = '0';
+                d[3] = '0';
+                d += 4;
+            }
+            d[0] = hex_chars_uc[c >> 4];
+            d[1] = hex_chars_uc[c & 0xF];
+            break;
+        }
+    }
+}
+
+
 /* decodes url-special-chars inplace.
  * replaces non-printable characters with '_'
  * (If this is used on a portion of query string, then query string should be
