@@ -34,6 +34,7 @@ enum http_header_h2_e { /* pseudo-headers */
  ,HTTP_HEADER_H2_PATH_INDEX_HTML = -6
  ,HTTP_HEADER_H2_SCHEME_HTTP     = -7
  ,HTTP_HEADER_H2_SCHEME_HTTPS    = -8
+ ,HTTP_HEADER_H2_PROTOCOL        = -9
 };
 enum http_header_e {
   HTTP_HEADER_OTHER = 0
@@ -76,6 +77,7 @@ enum http_header_e {
  ,HTTP_HEADER_ONION_LOCATION
  ,HTTP_HEADER_P3P
  ,HTTP_HEADER_PRAGMA
+ ,HTTP_HEADER_PRIORITY
  ,HTTP_HEADER_RANGE
  ,HTTP_HEADER_REFERER
  ,HTTP_HEADER_REFERRER_POLICY
