@@ -28,6 +28,7 @@ static const keyvalue http_versions[] = {
 static const buffer http_methods[] = {
 	{ CONST_STR_LEN("GET")+1, 0 },
 	{ CONST_STR_LEN("HEAD")+1, 0 },
+	{ CONST_STR_LEN("QUERY")+1, 0 },
 	{ CONST_STR_LEN("POST")+1, 0 },
 	{ CONST_STR_LEN("PUT")+1, 0 },
 	{ CONST_STR_LEN("DELETE")+1, 0 },
@@ -211,7 +212,7 @@ void http_status_append(buffer * const b, const int status) {
         buffer_append_string_len(b, kv->value, kv->vlen);
     else {
         buffer_append_int(b, status);
-        buffer_append_string_len(b, CONST_STR_LEN(" "));
+        buffer_append_char(b, ' ');
     }
 }
 
