@@ -29,7 +29,6 @@ typedef gw_handler_ctx   handler_ctx;
 #include "http_header.h"
 #include "http_kv.h"
 #include "log.h"
-#include "status_counter.h"
 
 #define AJP13_MAX_PACKET_SIZE 8192
 
@@ -644,7 +643,7 @@ ajp13_create_env (handler_ctx * const hctx)
         ajp13_stdin_append_n(hctx, AJP13_MAX_PACKET_SIZE-4);
         hctx->request_id = 0; /* overloaded value; see ajp13_stdin_append_n() */
 
-        status_counter_inc(CONST_STR_LEN("ajp13.requests"));
+        plugin_stats_inc("ajp13.requests");
         return HANDLER_GO_ON;
     } while (0);
 
@@ -694,7 +693,7 @@ ajp13_expand_headers (buffer * const b, handler_ctx * const hctx, uint32_t plen)
         ptr += 2;
         if (plen < len+1) break;
         plen -= len+1; /* include -1 for ending '\0' */
-        buffer_append_string_len(b, " ", 1);
+        buffer_append_char(b, ' ');
         if (len) buffer_append_string_len(b, (char *)ptr, len);
         ptr += len+1;
 
