@@ -6,14 +6,17 @@
 # include <crypt.h>
 #elif defined(__linux__)
 /* linux needs _XOPEN_SOURCE */
-# define _XOPEN_SOURCE
-#elif !defined(_MSC_VER)
-#include <unistd.h>
+#ifndef _XOPEN_SOURCE
+#define _XOPEN_SOURCE 700
 #endif
-
-#if defined(HAVE_LIBCRYPT) && !defined(HAVE_CRYPT)
-/* always assume crypt() is present if we have -lcrypt */
-# define HAVE_CRYPT
+#elif !defined(_MSC_VER)
+#ifndef _XOPEN_SOURCE
+#define _XOPEN_SOURCE 700
+#endif
+#ifndef _XOPEN_CRYPT
+#define _XOPEN_CRYPT 1
+#endif
+#include <unistd.h>
 #endif
 
 #include <stdlib.h>
