@@ -12,7 +12,6 @@
 #include "http_header.h"
 #include "log.h"
 #include "sock_addr.h"
-#include "status_counter.h"
 
 /**
  *
@@ -506,7 +505,7 @@ static void http_header_remap_uri (buffer *b, size_t off, http_header_remap_opts
         else {
             alen = buffer_clen(b) - off;
             if (0 == alen) return; /*(empty authority, e.g. "http:///")*/
-            buffer_append_string_len(b, CONST_STR_LEN("/"));
+            buffer_append_char(b, '/');
         }
 
         /* remap authority (if configured) and set offset to url-path */
@@ -611,18 +610,12 @@ static void buffer_append_string_backslash_escaped(buffer *b, const char *s, siz
 }
 
 static void proxy_set_Forwarded(connection * const con, request_st * const r, const unsigned int flags) {
-    buffer *b = NULL, *efor = NULL, *eproto = NULL, *ehost = NULL;
+    buffer *b = NULL;
+    buffer * const efor = (proxy_check_extforward)
+      ? http_header_env_get(r, CONST_STR_LEN("_L_EXTFORWARD_ACTUAL_FOR"))
+      : NULL;
     int semicolon = 0;
 
-    if (proxy_check_extforward) {
-        efor   =
-          http_header_env_get(r, CONST_STR_LEN("_L_EXTFORWARD_ACTUAL_FOR"));
-        eproto =
-          http_header_env_get(r, CONST_STR_LEN("_L_EXTFORWARD_ACTUAL_PROTO"));
-        ehost  =
-          http_header_env_get(r, CONST_STR_LEN("_L_EXTFORWARD_ACTUAL_HOST"));
-    }
-
     /* note: set "Forwarded" prior to updating X-Forwarded-For (below) */
 
     if (flags)
@@ -675,11 +668,11 @@ static void proxy_set_Forwarded(connection * const con, request_st * const r, co
             int ipv6 = (NULL != strchr(efor->ptr, ':'));
             ipv6
               ? buffer_append_string_len(b, CONST_STR_LEN("\"["))
-              : buffer_append_string_len(b, CONST_STR_LEN("\""));
+              : buffer_append_char(b, '"');
             buffer_append_string_backslash_escaped(b, BUF_PTR_LEN(efor));
             ipv6
               ? buffer_append_string_len(b, CONST_STR_LEN("]\""))
-              : buffer_append_string_len(b, CONST_STR_LEN("\""));
+              : buffer_append_char(b, '"');
         } else if (family == AF_INET) {
             /*(Note: if :port is added, then must be quoted-string:
              * e.g. for="...:port")*/
@@ -689,10 +682,10 @@ static void proxy_set_Forwarded(connection * const con, request_st * const r, co
                                   BUF_PTR_LEN(&con->dst_addr_buf),
                                   CONST_STR_LEN("]\""));
         } else {
-            buffer_append_string_len(b, CONST_STR_LEN("\""));
+            buffer_append_char(b, '"');
             buffer_append_string_backslash_escaped(
               b, BUF_PTR_LEN(&con->dst_addr_buf));
-            buffer_append_string_len(b, CONST_STR_LEN("\""));
+            buffer_append_char(b, '"');
         }
         semicolon = 1;
     }
@@ -705,7 +698,7 @@ static void proxy_set_Forwarded(connection * const con, request_st * const r, co
          *   INADDR_ANY or in6addr_any, but must omit optional :port
          *   from con->srv_socket->srv_token for consistency */
 
-        if (semicolon) buffer_append_string_len(b, CONST_STR_LEN(";"));
+        if (semicolon) buffer_append_char(b, ';');
         buffer_append_string_len(b, CONST_STR_LEN("by=\""));
       #ifdef HAVE_SYS_UN_H
         /* special-case: might need to encode unix domain socket path */
@@ -722,14 +715,17 @@ static void proxy_set_Forwarded(connection * const con, request_st * const r, co
                 sock_addr_stringify_append_buffer(b, &addr);
             }
         }
-        buffer_append_string_len(b, CONST_STR_LEN("\""));
+        buffer_append_char(b, '"');
         semicolon = 1;
     }
 
     if (flags & PROXY_FORWARDED_PROTO) {
+        const buffer * const eproto = (proxy_check_extforward)
+          ? http_header_env_get(r, CONST_STR_LEN("_L_EXTFORWARD_ACTUAL_PROTO"))
+          : NULL;
         /* expecting "http" or "https"
          * (not checking if quoted-string and encoding needed) */
-        if (semicolon) buffer_append_string_len(b, CONST_STR_LEN(";"));
+        if (semicolon) buffer_append_char(b, ';');
         if (NULL != eproto) {
             buffer_append_str2(b, CONST_STR_LEN("proto="), BUF_PTR_LEN(eproto));
         } else if (con->srv_socket->is_ssl) {
@@ -741,21 +737,24 @@ static void proxy_set_Forwarded(connection * const con, request_st * const r, co
     }
 
     if (flags & PROXY_FORWARDED_HOST) {
+        const buffer * const ehost = (proxy_check_extforward)
+          ? http_header_env_get(r, CONST_STR_LEN("_L_EXTFORWARD_ACTUAL_HOST"))
+          : NULL;
         if (NULL != ehost) {
             if (semicolon)
-                buffer_append_string_len(b, CONST_STR_LEN(";"));
+                buffer_append_char(b, ';');
             buffer_append_string_len(b, CONST_STR_LEN("host=\""));
             buffer_append_string_backslash_escaped(
               b, BUF_PTR_LEN(ehost));
-            buffer_append_string_len(b, CONST_STR_LEN("\""));
+            buffer_append_char(b, '"');
             semicolon = 1;
         } else if (r->http_host && !buffer_is_blank(r->http_host)) {
             if (semicolon)
-                buffer_append_string_len(b, CONST_STR_LEN(";"));
+                buffer_append_char(b, ';');
             buffer_append_string_len(b, CONST_STR_LEN("host=\""));
             buffer_append_string_backslash_escaped(
               b, BUF_PTR_LEN(r->http_host));
-            buffer_append_string_len(b, CONST_STR_LEN("\""));
+            buffer_append_char(b, '"');
             semicolon = 1;
         }
     }
@@ -765,11 +764,11 @@ static void proxy_set_Forwarded(connection * const con, request_st * const r, co
           http_header_env_get(r, CONST_STR_LEN("REMOTE_USER"));
         if (NULL != remote_user) {
             if (semicolon)
-                buffer_append_string_len(b, CONST_STR_LEN(";"));
+                buffer_append_char(b, ';');
             buffer_append_string_len(b, CONST_STR_LEN("remote_user=\""));
             buffer_append_string_backslash_escaped(
               b, BUF_PTR_LEN(remote_user));
-            buffer_append_string_len(b, CONST_STR_LEN("\""));
+            buffer_append_char(b, '"');
             /*semicolon = 1;*/
         }
     }
@@ -777,11 +776,11 @@ static void proxy_set_Forwarded(connection * const con, request_st * const r, co
     /* legacy X-* headers, including X-Forwarded-For */
 
     b = (NULL != efor) ? efor : &con->dst_addr_buf;
-    http_header_request_set(r, HTTP_HEADER_X_FORWARDED_FOR,
-                            CONST_STR_LEN("X-Forwarded-For"),
-                            BUF_PTR_LEN(b));
+    http_header_request_append(r, HTTP_HEADER_X_FORWARDED_FOR,
+                               CONST_STR_LEN("X-Forwarded-For"),
+                               BUF_PTR_LEN(b));
 
-    b = (NULL != ehost) ? ehost : r->http_host;
+    b = r->http_host;
     if (b && !buffer_is_blank(b)) {
         http_header_request_set(r, HTTP_HEADER_OTHER,
                                 CONST_STR_LEN("X-Host"),
@@ -791,7 +790,7 @@ static void proxy_set_Forwarded(connection * const con, request_st * const r, co
                                 BUF_PTR_LEN(b));
     }
 
-    b = (NULL != eproto) ? eproto : &r->uri.scheme;
+    b = &r->uri.scheme;
     http_header_request_set(r, HTTP_HEADER_X_FORWARDED_PROTO,
                             CONST_STR_LEN("X-Forwarded-Proto"),
                             BUF_PTR_LEN(b));
@@ -846,7 +845,10 @@ static handler_t proxy_create_env(gw_handler_ctx *gwhctx) {
 	/* build header */
 
 	/* request line */
-	const buffer * const m = http_method_buf(r->http_method);
+	const buffer * const m =
+	  http_method_buf(!r->h2_connect_ext
+			  ? r->http_method
+			  : HTTP_METHOD_GET); /*(translate HTTP/2 CONNECT ext)*/
 	buffer_append_str3(b,
 	                   BUF_PTR_LEN(m),
 	                   CONST_STR_LEN(" "),
@@ -891,10 +893,13 @@ static handler_t proxy_create_env(gw_handler_ctx *gwhctx) {
 			  r->reqbody_length);
 		}
 	}
-	else if (!hctx->conf.header.force_http10
-	         && -1 == r->reqbody_length
+	else if (r->h2_connect_ext) {
+	}
+	else if (-1 == r->reqbody_length
 	         && (r->conf.stream_request_body
 	             & (FDEVENT_STREAM_REQUEST | FDEVENT_STREAM_REQUEST_BUFMIN))) {
+		if (__builtin_expect( (hctx->conf.header.force_http10), 0))
+			return http_response_reqbody_read_error(r, 411);
 		hctx->gw.stdin_append = proxy_stdin_append; /* stream chunked body */
 		buffer_append_string_len(b, CONST_STR_LEN("\r\nTransfer-Encoding: chunked"));
 	}
@@ -930,7 +935,9 @@ static handler_t proxy_create_env(gw_handler_ctx *gwhctx) {
 			te = &ds->value; /*("trailers")*/
 			break;
 		case HTTP_HEADER_UPGRADE:
-			if (hctx->conf.header.force_http10 || r->http_version == HTTP_VERSION_1_0) continue;
+			if (hctx->conf.header.force_http10
+			    || (r->http_version != HTTP_VERSION_1_1 && !r->h2_connect_ext))
+				continue;
 			if (!hctx->conf.header.upgrade) continue;
 			if (!buffer_is_blank(&ds->value)) upgrade = &ds->value;
 			break;
@@ -993,6 +1000,24 @@ static handler_t proxy_create_env(gw_handler_ctx *gwhctx) {
 			buffer_append_string_len(b, CONST_STR_LEN(", upgrade"));
 		buffer_append_string_len(b, CONST_STR_LEN("\r\n\r\n"));
 	}
+	else if (r->h2_connect_ext) {
+		/* https://datatracker.ietf.org/doc/html/rfc6455#section-4.1
+		 * 7. The request MUST include a header field with the name
+		 *    |Sec-WebSocket-Key|.  The value of this header field MUST be a
+		 *    nonce consisting of a randomly selected 16-byte value that has
+		 *    been base64-encoded (see Section 4 of [RFC4648]).  The nonce
+		 *    MUST be selected randomly for each connection.
+		 * Note: Sec-WebSocket-Key is not used in RFC8441;
+		 *       include Sec-WebSocket-Key for HTTP/1.1 compatibility;
+		 *       !!not random!! base64-encoded "0000000000000000" */
+		if (!http_header_request_get(r, HTTP_HEADER_OTHER,
+		                             CONST_STR_LEN("Sec-WebSocket-Key")))
+			buffer_append_string_len(b, CONST_STR_LEN(
+			  "\r\nSec-WebSocket-Key: MDAwMDAwMDAwMDAwMDAwMAo="));
+		buffer_append_string_len(b, CONST_STR_LEN(
+		                              "\r\nUpgrade: websocket"
+		                              "\r\nConnection: close, upgrade\r\n\r\n"));
+	}
 	else    /* mod_proxy always sends Connection: close to backend */
 		buffer_append_string_len(b, CONST_STR_LEN("\r\nConnection: close\r\n\r\n"));
 
@@ -1010,7 +1035,7 @@ static handler_t proxy_create_env(gw_handler_ctx *gwhctx) {
 			chunkqueue_append_chunkqueue(&hctx->gw.wb, &r->reqbody_queue);
 	}
 
-	status_counter_inc(CONST_STR_LEN("proxy.requests"));
+	plugin_stats_inc("proxy.requests");
 	return HANDLER_GO_ON;
 }
 
@@ -1023,7 +1048,7 @@ static handler_t proxy_create_env_connect(gw_handler_ctx *gwhctx) {
 	gw_set_transparent(&hctx->gw);
 	http_response_upgrade_read_body_unknown(r);
 
-	status_counter_inc(CONST_STR_LEN("proxy.requests"));
+	plugin_stats_inc("proxy.requests");
 	return HANDLER_GO_ON;
 }
 
@@ -1036,6 +1061,13 @@ static handler_t proxy_response_headers(request_st * const r, struct http_respon
     if (light_btst(r->resp_htags, HTTP_HEADER_UPGRADE)) {
         if (remap_hdrs->upgrade && r->http_status == 101) {
             /* 101 Switching Protocols; transition to transparent proxy */
+            if (r->h2_connect_ext) {
+                r->http_status = 200; /* OK (response status for CONNECT) */
+                http_header_response_unset(r, HTTP_HEADER_UPGRADE,
+                                           CONST_STR_LEN("Upgrade"));
+                http_header_response_unset(r, HTTP_HEADER_OTHER,
+                                         CONST_STR_LEN("Sec-WebSocket-Accept"));
+            }
             gw_set_transparent(&hctx->gw);
             http_response_upgrade_read_body_unknown(r);
         }
@@ -1050,6 +1082,14 @@ static handler_t proxy_response_headers(request_st * const r, struct http_respon
           #endif
         }
     }
+    else if (__builtin_expect( (r->h2_connect_ext != 0), 0)
+             && r->http_status < 300) {
+        /*(not handling other 1xx intermediate responses here; not expected)*/
+        http_response_body_clear(r, 0);
+        r->handler_module = NULL;
+        r->http_status = 405; /* Method Not Allowed */
+        return HANDLER_FINISHED;
+    }
 
     /* rewrite paths, if needed */
 
@@ -1098,7 +1138,8 @@ static handler_t mod_proxy_check_extension(request_st * const r, void *p_d) {
 
 		hctx->conf = p->conf; /*(copies struct)*/
 		hctx->conf.header.http_host = r->http_host;
-		hctx->conf.header.upgrade  &= (r->http_version == HTTP_VERSION_1_1);
+		hctx->conf.header.upgrade  &=
+                  (r->http_version == HTTP_VERSION_1_1 || r->h2_connect_ext);
 		/* mod_proxy currently sends all backend requests as http.
 		 * https-remap is a flag since it might not be needed if backend
 		 * honors Forwarded or X-Forwarded-Proto headers, e.g. by using
@@ -1111,7 +1152,13 @@ static handler_t mod_proxy_check_extension(request_st * const r, void *p_d) {
 		if (r->http_method == HTTP_METHOD_CONNECT) {
 			/*(note: not requiring HTTP/1.1 due to too many non-compliant
 			 * clients such as 'openssl s_client')*/
-			if (hctx->conf.header.connect_method) {
+			if (r->h2_connect_ext
+			    && (hctx->conf.header.connect_method =
+			          hctx->conf.header.upgrade)) { /*(405 if not set)*/
+				/*(not bothering to check (!hctx->conf.header.force_http10))*/
+				/*hctx->gw.create_env = proxy_create_env;*/ /*(preserve)*/
+			}
+			else if (hctx->conf.header.connect_method) {
 				hctx->gw.create_env = proxy_create_env_connect;
 			}
 			else {
