@@ -14,7 +14,6 @@ typedef gw_handler_ctx   handler_ctx;
 #include "buffer.h"
 #include "http_cgi.h"
 #include "log.h"
-#include "status_counter.h"
 
 enum { LI_PROTOCOL_SCGI, LI_PROTOCOL_UWSGI };
 
@@ -209,11 +208,11 @@ static handler_t scgi_create_env(handler_ctx *hctx) {
 		scgi_env_add(b, CONST_STR_LEN("SCGI"), CONST_STR_LEN("1"));
 		buffer_clear(tb);
 		buffer_append_int(tb, buffer_clen(b)-10);
-		buffer_append_string_len(tb, CONST_STR_LEN(":"));
+		buffer_append_char(tb, ':');
 		len = buffer_clen(tb);
 		offset = 10 - len;
 		memcpy(b->ptr+offset, tb->ptr, len);
-		buffer_append_string_len(b, CONST_STR_LEN(","));
+		buffer_append_char(b, ',');
 	} else { /* LI_PROTOCOL_UWSGI */
 		/* http://uwsgi-docs.readthedocs.io/en/latest/Protocol.html */
 		size_t len = buffer_clen(b)-10;
@@ -245,7 +244,7 @@ static handler_t scgi_create_env(handler_ctx *hctx) {
 			hctx->wb_reqlen = -hctx->wb_reqlen;
 	}
 
-	status_counter_inc(CONST_STR_LEN("scgi.requests"));
+	plugin_stats_inc("scgi.requests");
 	return HANDLER_GO_ON;
 }
 
