@@ -1120,14 +1120,14 @@ static int process_ssi_stmt(request_st * const r, handler_ctx * const p, const c
 
 			buffer_append_str2(tb, BUF_PTR_LEN(&ds->key), CONST_STR_LEN("="));
 			buffer_append_string_encoded(tb, BUF_PTR_LEN(&ds->value), ENCODING_MINIMAL_XML);
-			buffer_append_string_len(tb, CONST_STR_LEN("\n"));
+			buffer_append_char(tb, '\n');
 		}
 		for (i = 0; i < p->ssi_cgi_env->used; i++) {
 			data_string *ds = (data_string *)p->ssi_cgi_env->sorted[i];
 
 			buffer_append_str2(tb, BUF_PTR_LEN(&ds->key), CONST_STR_LEN("="));
 			buffer_append_string_encoded(tb, BUF_PTR_LEN(&ds->value), ENCODING_MINIMAL_XML);
-			buffer_append_string_len(tb, CONST_STR_LEN("\n"));
+			buffer_append_char(tb, '\n');
 		}
 		chunkqueue_append_mem(cq, BUF_PTR_LEN(tb));
 		break;
@@ -1535,7 +1535,9 @@ static void mod_ssi_read_fd(request_st * const r, handler_ctx * const p, struct
 		 * (reduce occurrence of copying to reallocate larger chunk) */
 		if (cq->last && cq->last->type == MEM_CHUNK
 		    && buffer_string_space(cq->last->mem) < 1023)
-			http_chunk_transfer_cqlen(r, cq, chunkqueue_length(cq));
+			if (0 != http_chunk_transfer_cqlen(r, cq, chunkqueue_length(cq)))
+				chunkqueue_remove_empty_chunks(&r->write_queue);
+				/*(likely unrecoverable error if r->resp_send_chunked)*/
 	}
 
 	if (0 != rd) {
@@ -1551,7 +1553,9 @@ static void mod_ssi_read_fd(request_st * const r, handler_ctx * const p, struct
 	}
 
 	chunk_buffer_release(b);
-	http_chunk_transfer_cqlen(r, cq, chunkqueue_length(cq));
+	if (0 != http_chunk_transfer_cqlen(r, cq, chunkqueue_length(cq)))
+		chunkqueue_remove_empty_chunks(&r->write_queue);
+		/*(likely error unrecoverable if r->resp_send_chunked)*/
 }
 
 
