@@ -1,15 +1,15 @@
 #include "first.h"
 
-#include "base.h"
 #include "array.h"
 #include "buffer.h"
 #include "log.h"
+#include "request.h"
 #include "response.h"
+#include "stat_cache.h"
 
 #include "plugin.h"
 
 #include <sys/types.h>
-#include <sys/stat.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -31,12 +31,23 @@ typedef struct {
     PLUGIN_DATA;
     plugin_config defaults;
     plugin_config conf;
+    unix_time64_t cache_ts[2];
+    buffer cache_user[2];
+    buffer cache_path[2];
 } plugin_data;
 
 INIT_FUNC(mod_userdir_init) {
     return calloc(1, sizeof(plugin_data));
 }
 
+FREE_FUNC(mod_userdir_free) {
+    plugin_data * const p = p_d;
+    free(p->cache_user[0].ptr);
+    free(p->cache_user[1].ptr);
+    free(p->cache_path[0].ptr);
+    free(p->cache_path[1].ptr);
+}
+
 static void mod_userdir_merge_config_cpv(plugin_config * const pconf, const config_plugin_value_t * const cpv) {
     switch (cpv->k_id) { /* index into static config_plugin_keys_t cpk[] */
       case 0: /* userdir.path */
@@ -105,6 +116,29 @@ SETDEFAULTS_FUNC(mod_userdir_set_defaults) {
     if (!config_plugin_values_init(srv, p, cpk, "mod_userdir"))
         return HANDLER_ERROR;
 
+    /* process and validate config directives
+     * (init i to 0 if global context; to 1 to skip empty global context) */
+    for (int i = !p->cvlist[0].v.u2[1]; i < p->nconfig; ++i) {
+        config_plugin_value_t *cpv = p->cvlist + p->cvlist[i].v.u2[0];
+        for (; -1 != cpv->k_id; ++cpv) {
+            switch (cpv->k_id) {
+              case 0: /* userdir.path */
+              case 1: /* userdir.exclude-user */
+              case 2: /* userdir.include-user */
+                break;
+              case 3: /* userdir.basepath */
+                if (buffer_is_blank(cpv->v.b))
+                    cpv->v.b = NULL;
+                break;
+              case 4: /* userdir.letterhomes */
+              case 5: /* userdir.active */
+                break;
+              default:/* should not happen */
+                break;
+            }
+        }
+    }
+
     /* enabled by default for backward compatibility;
      * if userdir.path isn't set userdir is disabled too,
      * but you can't disable it by setting it to an empty string. */
@@ -147,17 +181,43 @@ static handler_t mod_userdir_docroot_construct(request_st * const r, plugin_data
     /* we build the physical path */
     buffer * const b = r->tmp_buf;
 
-    if (buffer_string_is_empty(p->conf.basepath)) {
+    if (!p->conf.basepath) {
       #ifdef HAVE_PWD_H
-        /* XXX: future: might add cache; getpwnam() lookup is expensive */
-        struct passwd *pwd = getpwnam(u);
-        if (pwd) {
-            struct stat st;
-            buffer_copy_string(b, pwd->pw_dir);
-            buffer_append_path_len(b, CONST_BUF_LEN(p->conf.path));
-            if (0 != stat(b->ptr, &st) || !S_ISDIR(st.st_mode)) {
+        /* getpwnam() lookup is expensive; first check 2-element cache */
+        const unix_time64_t cur_ts = log_monotonic_secs;
+        int cached = -1;
+        const int cache_sz =(int)(sizeof(p->cache_user)/sizeof(*p->cache_user));
+        for (int i = 0; i < cache_sz; ++i) {
+            if (cur_ts - p->cache_ts[i] < 60 && p->cache_user[i].used
+                && buffer_eq_slen(&p->cache_user[i], u, ulen)) {
+                cached = i;
+                break;
+            }
+        }
+        struct passwd *pwd;
+        if (cached >= 0) {
+            buffer_copy_path_len2(b, BUF_PTR_LEN(&p->cache_path[cached]),
+                                     BUF_PTR_LEN(p->conf.path));
+        }
+        else if ((pwd = getpwnam(u))) {
+            const size_t plen = strlen(pwd->pw_dir);
+            buffer_copy_path_len2(b, pwd->pw_dir, plen,
+                                     BUF_PTR_LEN(p->conf.path));
+            if (!stat_cache_path_isdir(b)) {
                 return HANDLER_GO_ON;
             }
+            /* update cache, replacing oldest entry */
+            cached = 0;
+            unix_time64_t cache_ts = p->cache_ts[0];
+            for (int i = 1; i < cache_sz; ++i) {
+                if (cache_ts > p->cache_ts[i]) {
+                    cache_ts = p->cache_ts[i];
+                    cached = i;
+                }
+            }
+            p->cache_ts[cached] = cur_ts;
+            buffer_copy_string_len(&p->cache_path[cached], b->ptr, plen);
+            buffer_copy_string_len(&p->cache_user[cached], u, ulen);
         }
         else /* user not found */
       #endif
@@ -189,7 +249,7 @@ static handler_t mod_userdir_docroot_construct(request_st * const r, plugin_data
             buffer_append_path_len(b, u, 1);
         }
         buffer_append_path_len(b, u, ulen);
-        buffer_append_path_len(b, CONST_BUF_LEN(p->conf.path));
+        buffer_append_path_len(b, BUF_PTR_LEN(p->conf.path));
     }
 
     buffer_copy_buffer(&r->physical.basedir, b);
@@ -223,7 +283,7 @@ URIHANDLER_FUNC(mod_userdir_docroot_handler) {
     /* /~user/foo.html -> /home/user/public_html/foo.html */
 
   #ifdef __COVERITY__
-    if (buffer_is_empty(&r->uri.path)) return HANDLER_GO_ON;
+    if (buffer_is_blank(&r->uri.path)) return HANDLER_GO_ON;
   #endif
 
     if (r->uri.path.ptr[0] != '/' ||
@@ -235,7 +295,7 @@ URIHANDLER_FUNC(mod_userdir_docroot_handler) {
     /* enforce the userdir.path to be set in the config, ugly fix for #1587;
      * should be replaced with a clean .enabled option in 1.5
      */
-    if (!p->conf.active || buffer_is_empty(p->conf.path)) return HANDLER_GO_ON;
+    if (!p->conf.active || !p->conf.path) return HANDLER_GO_ON;
 
     const char * const uptr = r->uri.path.ptr + 2;
     const char * const rel_url = strchr(uptr, '/');
@@ -278,6 +338,7 @@ int mod_userdir_plugin_init(plugin *p) {
 	p->name        = "userdir";
 
 	p->init           = mod_userdir_init;
+	p->cleanup        = mod_userdir_free;
 	p->handle_physical = mod_userdir_docroot_handler;
 	p->set_defaults   = mod_userdir_set_defaults;
 
