@@ -74,6 +74,13 @@ static handler_t network_server_handle_fdevent(void *context, int revents) {
 
         if (nagle_disable)
             network_accept_tcp_nagle_disable(fd);
+      #ifdef HAVE_SYS_UN_H /*(see sock_addr.h)*/
+        else if (addrlen <= 2) /*(AF_UNIX if !nagle_disable)*/
+            memcpy(addr.un.sun_path, srv_socket->addr.un.sun_path,
+                   srv_socket->srv_token_colon < sizeof(addr.un.sun_path)
+                   ? (size_t)srv_socket->srv_token_colon+1 /*(+1 for '\0')*/
+                   : sizeof(addr.un.sun_path));/*(escaped len might be longer)*/
+      #endif
 
         connection *con = connection_accepted(srv, srv_socket, &addr, fd);
         if (__builtin_expect( (!con), 0)) return HANDLER_GO_ON;
@@ -98,9 +105,103 @@ static handler_t network_server_handle_fdevent(void *context, int revents) {
     return HANDLER_GO_ON;
 }
 
-static void network_host_normalize_addr_str(buffer *host, sock_addr *addr) {
+#ifdef HAVE_SYS_UN_H
+
+/* abstract socket (Linux, QNX?, Windows 10?) */
+
+__attribute_cold__
+static void
+network_abstract_socket_enc (buffer * const restrict abstract,
+                             unsigned char * const restrict sun_path,
+                             const uint32_t len)
+{
+    /*(strings needing encoding are expected to be short;
+     * code not written for performance)*/
+    buffer_clear(abstract);
+    uint32_t n = 0;
+    for (uint32_t i = 0; i < len; ++i)
+        n += (sun_path[i]-20 < 107) ? 1 : 4;
+    char *s = buffer_extend(abstract, n);
+    for (uint32_t i = 0; i < len; ++i) {
+        /* (sun_path[i] >= 20 && sun_path[i] < 127 && sun_path[i] != '\\') */
+        if (sun_path[i]-20 < 107 && sun_path[i] != '\\')
+            *s++ = ((char *)sun_path)[i];
+        else {
+            s[0] = '\\';
+            s[1] = 'x';
+            s[2] = "0123456789abcdef"[sun_path[i] >>  4];
+            s[3] = "0123456789abcdef"[sun_path[i] & 0xF];
+            s += 4;
+        }
+    }
+}
+
+__attribute_cold__
+static uint32_t
+network_abstract_socket_dec (const buffer * const restrict abstract,
+                             char * const restrict sun_path,
+                             const uint32_t plen)
+{
+    /*(strings expected to begin with "\\x00")*/
+    /*(strings needing decoding are expected to be short;
+     * code not written for performance)*/
+    const char *s = abstract->ptr;
+    uint32_t n = 0;
+    for (int hi, lo; *s && n < plen; ++n) {
+        if (s[0] != '\\')
+            sun_path[n] = *s++;
+        else if (s[1] == 'x'
+                 && (hi = hex2int(s[2])) != 0xFF
+                 && (lo = hex2int(s[3])) != 0xFF) {
+            sun_path[n] = (char)((hi << 4) | lo);
+            s += 4;
+        }
+        else
+            break;
+    }
+    return *s == '\0' ? n : 0;
+}
+
+__attribute_cold__
+static int
+network_abstract_socket_parse (const buffer *abstract,
+                               sock_addr *addr, socklen_t *addr_len,
+                               log_error_st *errh)
+{
+    /* abstract socket (Linux, QNX?, Windows 10?) */
+    /*assert(*addr_len >= sizeof(struct sockaddr_un));*/
+    memset(addr, 0, sizeof(struct sockaddr_un));
+    addr->un.sun_family = AF_UNIX;
+    uint32_t len =
+      (uint32_t)(*addr_len - offsetof(struct sockaddr_un, sun_path));
+    if (len > sizeof(addr->un.sun_path))
+        len = sizeof(addr->un.sun_path);
+    len = network_abstract_socket_dec(abstract, addr->un.sun_path, len);
+    if (len) {
+        *addr_len = offsetof(struct sockaddr_un, sun_path) + len;
+        return 0;
+    }
+    else {
+        log_error(errh, __FILE__, __LINE__,
+          "abstract unix socket filename invalid encoding or too long: %s",
+          abstract->ptr);
+        return -1;
+    }
+}
+
+#endif /* HAVE_SYS_UN_H */
+
+static void network_host_normalize_addr_str(buffer *host, sock_addr *addr, socklen_t addr_len) {
     buffer_clear(host);
     sock_addr_stringify_append_buffer(host, addr);
+  #ifdef HAVE_SYS_UN_H
+    if (AF_UNIX == sock_addr_get_family(addr) && 0 == buffer_clen(host))
+        network_abstract_socket_enc(host, (unsigned char *)addr->un.sun_path,
+                                    (uint32_t)(addr_len
+                                    - offsetof(struct sockaddr_un, sun_path)));
+  #else
+    UNUSED(addr_len);
+  #endif
 }
 
 static int network_host_parse_addr(server *srv, sock_addr *addr, socklen_t *addr_len, buffer *host, int use_ipv6) {
@@ -115,14 +216,16 @@ static int network_host_parse_addr(server *srv, sock_addr *addr, socklen_t *addr
         return -1;
     }
     h = host->ptr;
-    if (h[0] == '/') {
+    if (h[0] == '/' || h[0] == '\\') {
       #ifdef HAVE_SYS_UN_H
+        if (h[0] == '\\' && h[1] == 'x' && h[2] == '0' && h[3] == '0')
+            return network_abstract_socket_parse(host,addr,addr_len,srv->errh);
         return (1 ==
                 sock_addr_from_str_hints(addr,addr_len,h,AF_UNIX,0,srv->errh))
           ? 0
           : -1;
       #else
-        log_error(srv, __FILE__, __LINE__,
+        log_error(srv->errh, __FILE__, __LINE__,
           "ERROR: Unix Domain sockets are not supported.");
         return -1;
       #endif
@@ -241,12 +344,25 @@ static uint8_t network_srv_token_colon (const buffer * const b) {
     return colon ? (uint8_t)(colon - p) : (uint8_t)buffer_clen(b);
 }
 
-static int network_server_init(server *srv, network_socket_config *s, buffer *host_token, size_t sidx, int stdin_fd) {
+static void network_srv_socket_init_token (server_socket * const srv_socket, const buffer * const token) {
+    buffer * const srv_token = srv_socket->srv_token = buffer_init();
+    buffer_copy_buffer(srv_token, token);
+  #ifdef HAVE_SYS_UN_H
+    /*(srv_socket->addr must have been initialized by caller)*/
+    if (AF_UNIX == sock_addr_get_family(&srv_socket->addr))
+        srv_socket->srv_token_colon = buffer_clen(srv_token);
+    else
+  #endif
+        srv_socket->srv_token_colon = network_srv_token_colon(srv_token);
+}
+
+static int network_server_init(server *srv, const network_socket_config *s, buffer *host_token, size_t sidx, int stdin_fd) {
 	server_socket *srv_socket;
 	const char *host;
 	socklen_t addr_len = sizeof(sock_addr);
 	sock_addr addr;
 	int family = 0;
+	int use_ipv6 = s->use_ipv6;
 	int set_v6only = 0;
 
 	if (buffer_is_blank(host_token)) {
@@ -265,13 +381,13 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 	}
 
 	host = host_token->ptr;
-	if ((s->use_ipv6 && (*host == '\0' || *host == ':')) || (host[0] == '[' && host[1] == ']')) {
+	if ((use_ipv6 && (*host == '\0' || *host == ':')) || (host[0] == '[' && host[1] == ']')) {
 		log_error(srv->errh, __FILE__, __LINE__,
 		  "warning: please use server.use-ipv6 only for hostnames, "
 		  "not without server.bind / empty address; your config will "
 		  "break if the kernel default for IPV6_V6ONLY changes");
 	}
-	if (*host == '[') s->use_ipv6 = 1;
+	if (*host == '[') use_ipv6 = 1;
 
 	memset(&addr, 0, sizeof(addr));
 	if (-1 != stdin_fd) {
@@ -279,7 +395,7 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 			log_perror(srv->errh, __FILE__, __LINE__, "getsockname()");
 			return -1;
 		}
-	} else if (0 != network_host_parse_addr(srv, &addr, &addr_len, host_token, s->use_ipv6)) {
+	} else if (0 != network_host_parse_addr(srv, &addr, &addr_len, host_token, use_ipv6)) {
 		return -1;
 	}
 
@@ -300,7 +416,7 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 	}
       #endif
 
-	network_host_normalize_addr_str(host_token, &addr);
+	network_host_normalize_addr_str(host_token, &addr, addr_len);
 	host = host_token->ptr;
 
 	if (srv->srvconf.preflight_check) {
@@ -310,6 +426,10 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 	/* check if we already know this socket (after potential DNS resolution), and if yes, don't init it */
 	for (uint32_t i = 0; i < srv->srv_sockets.used; ++i) {
 		if (0 == memcmp(&srv->srv_sockets.ptr[i]->addr, &addr, sizeof(addr))) {
+			if ((unsigned short)~0u == srv->srv_sockets.ptr[i]->sidx) {
+				srv->srv_sockets.ptr[i]->sidx = sidx;
+				srv->srv_sockets.ptr[i]->is_ssl = s->ssl_enabled;
+			}
 			return 0;
 		}
 	}
@@ -321,10 +441,7 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 	srv_socket->sidx = sidx;
 	srv_socket->is_ssl = s->ssl_enabled;
 	srv_socket->srv = srv;
-	buffer_copy_buffer((srv_socket->srv_token = buffer_init()), host_token);
-	srv_socket->srv_token_colon =
-	  network_srv_token_colon(srv_socket->srv_token);
-
+	network_srv_socket_init_token(srv_socket, host_token);
 	network_srv_sockets_append(srv, srv_socket);
 
 	if (srv->sockets_disabled) { /* lighttpd -1 (one-shot mode) */
@@ -366,7 +483,7 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 		/* connect failed */
 		switch(errno) {
 		case ECONNREFUSED:
-			unlink(host);
+			if (*host == '/') unlink(host);
 			break;
 		case ENOENT:
 			break;
@@ -421,6 +538,7 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 		return -1;
 	}
 
+  #ifdef HAVE_SYS_UN_H
 	if (-1 != stdin_fd) { } else
 	if (AF_UNIX == family && s->socket_perms) {
 		mode_t m = 0;
@@ -428,12 +546,13 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 			m <<= 3;
 			m |= (*str - '0');
 		}
-		if (0 != m && -1 == chmod(host, m)) {
+		if (0 != m && *host == '/' && -1 == chmod(host, m)) {
 			log_perror(srv->errh, __FILE__, __LINE__,
 			  "chmod(\"%s\", %s)", host, s->socket_perms->ptr);
 			return -1;
 		}
 	}
+  #endif
 
 	if (-1 != stdin_fd) { } else
 	if (-1 == listen(srv_socket->fd, s->listen_backlog)) {
@@ -552,7 +671,7 @@ void network_socket_activation_to_env (server * const srv) {
     setenv("LISTEN_PID", tb->ptr, 1);
 }
 
-static int network_socket_activation_nfds(server *srv, network_socket_config *s, int nfds) {
+static int network_socket_activation_nfds(server *srv, const network_socket_config *s, int nfds) {
     buffer *host = buffer_init();
     socklen_t addr_len;
     sock_addr addr;
@@ -565,7 +684,7 @@ static int network_socket_activation_nfds(server *srv, network_socket_config *s,
               "socket activation getsockname()");
             break;
         }
-        network_host_normalize_addr_str(host, &addr);
+        network_host_normalize_addr_str(host, &addr, addr_len);
         rc = network_server_init(srv, s, host, 0, fd);
         if (0 != rc) break;
         srv->srv_sockets.ptr[srv->srv_sockets.used-1]->sidx = (unsigned short)~0u;
@@ -576,7 +695,7 @@ static int network_socket_activation_nfds(server *srv, network_socket_config *s,
     return rc;
 }
 
-static int network_socket_activation_from_env(server *srv, network_socket_config *s) {
+static int network_socket_activation_from_env(server *srv, const network_socket_config *s) {
     char *listen_pid = getenv("LISTEN_PID");
     char *listen_fds = getenv("LISTEN_FDS");
     pid_t lpid = listen_pid ? (pid_t)strtoul(listen_pid,NULL,10) : 0;
@@ -602,33 +721,28 @@ int network_init(server *srv, int stdin_fd) {
     static const config_plugin_keys_t cpk[] = {
       { CONST_STR_LEN("ssl.engine"),
         T_CONFIG_BOOL,
-        T_CONFIG_SCOPE_CONNECTION }
+        T_CONFIG_SCOPE_SOCKET }
      ,{ CONST_STR_LEN("server.listen-backlog"),
         T_CONFIG_INT,
-        T_CONFIG_SCOPE_CONNECTION }
+        T_CONFIG_SCOPE_SOCKET }
      ,{ CONST_STR_LEN("server.socket-perms"),
         T_CONFIG_STRING,
-        T_CONFIG_SCOPE_CONNECTION }
+        T_CONFIG_SCOPE_SOCKET }
      ,{ CONST_STR_LEN("server.bsd-accept-filter"),
         T_CONFIG_STRING,
-        T_CONFIG_SCOPE_CONNECTION }
+        T_CONFIG_SCOPE_SOCKET }
      ,{ CONST_STR_LEN("server.defer-accept"),
         T_CONFIG_BOOL,
-        T_CONFIG_SCOPE_CONNECTION }
+        T_CONFIG_SCOPE_SOCKET }
      ,{ CONST_STR_LEN("server.use-ipv6"),
         T_CONFIG_BOOL,
-        T_CONFIG_SCOPE_CONNECTION }
+        T_CONFIG_SCOPE_SOCKET }
      ,{ CONST_STR_LEN("server.set-v6only"),
         T_CONFIG_BOOL,
-        T_CONFIG_SCOPE_CONNECTION }
+        T_CONFIG_SCOPE_SOCKET }
      ,{ CONST_STR_LEN("server.v4mapped"),
         T_CONFIG_BOOL,
-        T_CONFIG_SCOPE_CONNECTION }
-    #if 0 /* TODO: more integration needed ... */
-     ,{ CONST_STR_LEN("mbedtls.engine"),
-        T_CONFIG_BOOL,
-        T_CONFIG_SCOPE_CONNECTION }
-    #endif
+        T_CONFIG_SCOPE_SOCKET }
      ,{ NULL, 0,
         T_CONFIG_UNSET,
         T_CONFIG_SCOPE_UNSET }
@@ -720,7 +834,7 @@ int network_init(server *srv, int stdin_fd) {
                 rc = network_host_parse_addr(srv, &addr, &addr_len,
                                              host_token, p->conf.use_ipv6);
                 if (0 != rc) break;
-                network_host_normalize_addr_str(host_token, &addr);
+                network_host_normalize_addr_str(host_token, &addr, addr_len);
             }
         }
         if (0 != rc) break;
@@ -739,8 +853,8 @@ int network_init(server *srv, int stdin_fd) {
             if (srv->srvconf.bindhost)
                 buffer_copy_buffer(b, srv->srvconf.bindhost);
             /*(skip adding port if unix socket path)*/
-            if (!b->ptr || b->ptr[0] != '/') {
-                buffer_append_string_len(b, CONST_STR_LEN(":"));
+            if (!b->ptr || (b->ptr[0] != '/' && b->ptr[0] != '\\')) {
+                buffer_append_char(b, ':');
                 buffer_append_int(b, srv->srvconf.port);
             }
           #ifdef __COVERITY__
@@ -764,15 +878,21 @@ int network_init(server *srv, int stdin_fd) {
                 force_assert(NULL != srv_socket);
                 memcpy(srv_socket, srv->srv_sockets_inherited.ptr[i],
                        sizeof(server_socket));
-                const buffer * const srv_token = srv_socket->srv_token;
-                buffer_copy_buffer((srv_socket->srv_token = buffer_init()),
-                                   srv_token);
-                srv_socket->srv_token_colon =
-                  network_srv_token_colon(srv_socket->srv_token);
+                srv_socket->is_ssl = p->defaults.ssl_enabled;
+                /*(note: re-inits srv_socket->srv_token to new buffer ptr)*/
+                network_srv_socket_init_token(srv_socket,srv_socket->srv_token);
                 network_srv_sockets_append(srv, srv_socket);
             }
         }
 
+        /* reset sidx of any graceful sockets not explicitly listed in config */
+        for (uint32_t i = 0; i < srv->srv_sockets.used; ++i) {
+            if ((unsigned short)~0u == srv->srv_sockets.ptr[i]->sidx) {
+                srv->srv_sockets.ptr[i]->sidx = 0;
+                srv->srv_sockets.ptr[i]->is_ssl = p->defaults.ssl_enabled;
+            }
+        }
+
     } while (0);
 
     free(p->cvlist);
