@@ -15,6 +15,7 @@
 #include "http_range.h"
 
 #include "plugin.h"
+#include "plugins.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -26,6 +27,18 @@
 #include <string.h>
 
 
+__attribute_cold__
+void
+http_response_delay (connection * const con)
+{
+    if (config_feature_bool(con->srv, "auth.delay-invalid-creds", 1)){
+        /*(delay sending response)*/
+        con->is_writable = 0;
+        con->traffic_limit_reached = 1;
+    }
+}
+
+
 int
 http_response_omit_header (request_st * const r, const data_string * const ds)
 {
@@ -96,6 +109,8 @@ http_response_write_header (request_st * const r)
 	    && r->http_version == HTTP_VERSION_1_1) {
 		http_header_response_set(r, HTTP_HEADER_CONNECTION, CONST_STR_LEN("Connection"), CONST_STR_LEN("upgrade"));
 	} else if (r->keep_alive <= 0) {
+		if (r->keep_alive < 0)
+			http_response_delay(r->con);
 		http_header_response_set(r, HTTP_HEADER_CONNECTION, CONST_STR_LEN("Connection"), CONST_STR_LEN("close"));
 	} else if (r->http_version == HTTP_VERSION_1_0) {/*(&& r->keep_alive > 0)*/
 		http_header_response_set(r, HTTP_HEADER_CONNECTION, CONST_STR_LEN("Connection"), CONST_STR_LEN("keep-alive"));
@@ -350,8 +365,11 @@ static handler_t http_response_config (request_st * const r) {
 
     /* do we have to downgrade from 1.1 to 1.0 ? (ignore for HTTP/2) */
     if (__builtin_expect( (!r->conf.allow_http11), 0)
-        && r->http_version == HTTP_VERSION_1_1)
+        && r->http_version == HTTP_VERSION_1_1) {
         r->http_version = HTTP_VERSION_1_0;
+        /*(when forcing HTTP/1.0, ignore (unlikely) Connection: keep-alive)*/
+        r->keep_alive = 0;
+    }
 
     if (__builtin_expect( (r->reqbody_length > 0), 0)
         && 0 != r->conf.max_request_size   /* r->conf.max_request_size in kB */
@@ -366,19 +384,14 @@ static handler_t http_response_config (request_st * const r) {
 }
 
 
-__attribute_cold__
-static handler_t http_response_comeback (request_st * const r);
-
-
+__attribute_noinline__
 static handler_t
 http_response_prepare (request_st * const r)
 {
     handler_t rc;
 
-    do {
-
 	/* looks like someone has already made a decision */
-	if (r->http_status != 0 && r->http_status != 200) {
+	if (__builtin_expect( (r->http_status > 200), 0)) { /* yes, > 200 */
 		if (0 == r->resp_body_finished)
 			http_response_body_clear(r, 0);
 		return HANDLER_FINISHED;
@@ -389,7 +402,7 @@ http_response_prepare (request_st * const r)
 
 		if (__builtin_expect( (!r->async_callback), 1)) {
 			rc = http_response_config(r);
-			if (HANDLER_GO_ON != rc) continue;
+			if (HANDLER_GO_ON != rc) return rc;
 		}
 		else
 			r->async_callback = 0; /* reset */
@@ -421,13 +434,14 @@ http_response_prepare (request_st * const r)
 
 
 		rc = plugins_call_handle_uri_clean(r);
-		if (HANDLER_GO_ON != rc) continue;
+		if (HANDLER_GO_ON != rc) return rc;
 
 		if (__builtin_expect( (r->http_method == HTTP_METHOD_OPTIONS), 0)
 		    && r->uri.path.ptr[0] == '*' && r->uri.path.ptr[1] == '\0')
 			return http_response_prepare_options_star(r);
 
-		if (__builtin_expect( (r->http_method == HTTP_METHOD_CONNECT), 0))
+		if (__builtin_expect( (r->http_method == HTTP_METHOD_CONNECT), 0)
+		    && (r->handler_module || !r->h2_connect_ext))
 			return http_response_prepare_connect(r);
 
 
@@ -441,7 +455,7 @@ http_response_prepare (request_st * const r)
 		buffer_clear(&r->physical.doc_root);
 
 		rc = plugins_call_handle_docroot(r);
-		if (HANDLER_GO_ON != rc) continue;
+		if (HANDLER_GO_ON != rc) return rc;
 
 
 		/* transform r->uri.path to r->physical.rel_path (relative file path) */
@@ -489,7 +503,7 @@ http_response_prepare (request_st * const r)
 		                      BUF_PTR_LEN(&r->physical.rel_path));
 
 			rc = plugins_call_handle_physical(r);
-			if (HANDLER_GO_ON != rc) continue;
+			if (HANDLER_GO_ON != rc) return rc;
 
 			if (r->conf.log_request_handling) {
 				log_error(r->conf.errh, __FILE__, __LINE__,
@@ -514,7 +528,7 @@ http_response_prepare (request_st * const r)
 	 */
 
 		rc = http_response_physical_path_check(r);
-		if (HANDLER_GO_ON != rc) continue;
+		if (HANDLER_GO_ON != rc) return rc;
 
 		/* r->physical.path is non-empty and exists in the filesystem */
 
@@ -533,24 +547,33 @@ http_response_prepare (request_st * const r)
 
 		/* call the handlers */
 		rc = plugins_call_handle_subrequest_start(r);
-		if (HANDLER_GO_ON != rc) continue;
+		if (HANDLER_GO_ON != rc) return rc;
 
-		/* if we are still here, no one wanted the file, status 403 is ok I think */
-		if (NULL == r->handler_module && 0 == r->http_status) {
-			r->http_status = (r->http_method != HTTP_METHOD_OPTIONS) ? 403 : 200;
+		if (__builtin_expect( (NULL == r->handler_module), 0)) {
+			/* no handler; finish request */
+			if (__builtin_expect( (0 == r->http_status), 0)) {
+				if (r->http_method == HTTP_METHOD_OPTIONS) {
+					http_response_body_clear(r, 0);
+					http_response_prepare_options_star(r); /*(treat like "*")*/
+				}
+				else if (r->http_method == HTTP_METHOD_CONNECT)
+					/* 405 Method Not Allowed */
+					return http_status_set_error_close(r, 405);
+					/*return http_response_prepare_connect(r);*/
+				else if (!http_method_get_head_query_post(r->http_method))
+					r->http_status = 501;
+				else
+					r->http_status = 403;
+			}
 			return HANDLER_FINISHED;
 		}
 
 		return HANDLER_GO_ON;
-
-    } while (HANDLER_COMEBACK == rc
-             && HANDLER_GO_ON ==(rc = http_response_comeback(r)));
-
-    return rc;
 }
 
 
 __attribute_cold__
+__attribute_noinline__
 static handler_t http_response_comeback (request_st * const r)
 {
     if (NULL != r->handler_module || !buffer_is_unset(&r->physical.path))
@@ -596,7 +619,7 @@ http_response_errdoc_init (request_st * const r)
         const buffer * const vb =
           http_header_response_get(r, HTTP_HEADER_WWW_AUTHENTICATE,
                                    CONST_STR_LEN("WWW-Authenticate"));
-        if (NULL != vb) www_auth = buffer_init_buffer(vb);
+        if (NULL != vb) buffer_copy_buffer((www_auth = buffer_init()), vb);
     }
 
     buffer_reset(&r->physical.path);
@@ -710,31 +733,10 @@ http_response_merge_trailers (request_st * const r)
 }
 
 
+__attribute_noinline__
 static handler_t
 http_response_write_prepare(request_st * const r)
 {
-    if (NULL == r->handler_module) {
-        /* static files */
-        switch(r->http_method) {
-          case HTTP_METHOD_GET:
-          case HTTP_METHOD_POST:
-          case HTTP_METHOD_HEAD:
-            break;
-          case HTTP_METHOD_OPTIONS:
-            if ((!r->http_status || r->http_status == 200)
-                && !buffer_is_blank(&r->uri.path)
-                && r->uri.path.ptr[0] != '*') {
-                http_response_body_clear(r, 0);
-		http_response_prepare_options_star(r); /*(treat like "*")*/
-            }
-            break;
-          default:
-            if (r->http_status == 0)
-                r->http_status = 501;
-            break;
-        }
-    }
-
     switch (r->http_status) {
       case 200: /* common case */
         break;
@@ -749,8 +751,6 @@ http_response_write_prepare(request_st * const r)
         r->resp_body_finished = 1;
         break;
       default: /* class: header + body */
-        if (r->http_status == 0)
-            r->http_status = 403;
         /* only custom body for 4xx and 5xx */
         if (r->http_status >= 400 && r->http_status < 600)
             http_response_static_errdoc(r);
@@ -882,7 +882,7 @@ http_response_write_prepare(request_st * const r)
 
 
 __attribute_cold__
-static handler_t
+static int
 http_response_call_error_handler (request_st * const r, const buffer * const error_handler)
 {
     /* call error-handler */
@@ -924,16 +924,57 @@ http_response_call_error_handler (request_st * const r, const buffer * const err
     buffer_copy_buffer(&r->target, error_handler);
     http_response_errdoc_init(r);
     r->http_status = 0; /*(after http_response_errdoc_init())*/
-    http_response_comeback(r);
-    return HANDLER_COMEBACK;
+    return 1;
+}
+
+
+__attribute_cold__
+__attribute_noinline__
+static int
+http_response_has_error_handler (request_st * const r)
+{
+    if (r->error_handler_saved_status > 0)
+        r->http_method = r->error_handler_saved_method;
+    if (NULL == r->handler_module || r->conf.error_intercept) {
+        if (__builtin_expect( (r->error_handler_saved_status), 0)) {
+            const int subreq_status = r->http_status;
+            if (r->error_handler_saved_status > 0)
+                r->http_status = r->error_handler_saved_status;
+            else if (r->http_status == 404 || r->http_status == 403)
+                /* error-handler-404 is a 404 */
+                r->http_status = -r->error_handler_saved_status;
+            else {
+                /* error-handler-404 is back and has generated content */
+                /* if Status: was set, take it otherwise use 200 */
+            }
+            if (200 <= subreq_status && subreq_status <= 299) {
+                /*(flag value to indicate that error handler succeeded)
+                 *(for (NULL == r->handler_module))*/
+                r->error_handler_saved_status = 65535; /* >= 1000 */
+            }
+        }
+        else if (__builtin_expect( (r->http_status >= 400), 0)) {
+            const buffer *error_handler = NULL;
+            if (r->conf.error_handler)
+                error_handler = r->conf.error_handler;
+            else if ((r->http_status == 404 || r->http_status == 403)
+                   && r->conf.error_handler_404)
+                error_handler = r->conf.error_handler_404;
+
+            if (error_handler)
+                return http_response_call_error_handler(r, error_handler);
+        }
+    }
+    return 0;
 }
 
 
 handler_t
 http_response_handler (request_st * const r)
 {
+  int rc;
+  do {
     const plugin *p = r->handler_module;
-    int rc;
     if (NULL != p
         || ((rc = http_response_prepare(r)) == HANDLER_GO_ON
             && NULL != (p = r->handler_module)))
@@ -950,49 +991,25 @@ http_response_handler (request_st * const r)
         /* response headers received from backend; start response */
         __attribute_fallthrough__
       case HANDLER_GO_ON:
-      case HANDLER_FINISHED: /*(HANDLER_FINISHED if request not handled)*/
+      case HANDLER_FINISHED:
         if (r->http_status == 0) r->http_status = 200;
-        if (r->error_handler_saved_status > 0)
-            r->http_method = r->error_handler_saved_method;
-        if (NULL == r->handler_module || r->conf.error_intercept) {
-            if (__builtin_expect( (r->error_handler_saved_status), 0)) {
-                const int subreq_status = r->http_status;
-                if (r->error_handler_saved_status > 0)
-                    r->http_status = r->error_handler_saved_status;
-                else if (r->http_status == 404 || r->http_status == 403)
-                    /* error-handler-404 is a 404 */
-                    r->http_status = -r->error_handler_saved_status;
-                else {
-                    /* error-handler-404 is back and has generated content */
-                    /* if Status: was set, take it otherwise use 200 */
-                }
-                if (200 <= subreq_status && subreq_status <= 299) {
-                    /*(flag value to indicate that error handler succeeded)
-                     *(for (NULL == r->handler_module))*/
-                    r->error_handler_saved_status = 65535; /* >= 1000 */
-                }
-            }
-            else if (__builtin_expect( (r->http_status >= 400), 0)) {
-                const buffer *error_handler = NULL;
-                if (r->conf.error_handler)
-                    error_handler = r->conf.error_handler;
-                else if ((r->http_status == 404 || r->http_status == 403)
-                       && r->conf.error_handler_404)
-                    error_handler = r->conf.error_handler_404;
-
-                if (error_handler)
-                    return http_response_call_error_handler(r, error_handler);
-            }
-        }
-
-        /* we have something to send; go on */
-        /*(CON_STATE_RESPONSE_START; transient state)*/
-        return http_response_write_prepare(r);
+        if ((__builtin_expect( (r->http_status < 400), 1)
+             && __builtin_expect( (0 == r->error_handler_saved_status), 1))
+            || __builtin_expect( (!http_response_has_error_handler(r)), 1))
+            /* we have something to send; go on */
+            /*(CON_STATE_RESPONSE_START; transient state)*/
+            return http_response_write_prepare(r);
+        __attribute_fallthrough__
       case HANDLER_COMEBACK:
         http_response_comeback(r);
-        return HANDLER_COMEBACK;
+        rc = HANDLER_COMEBACK;
+        continue;
       /*case HANDLER_ERROR:*/
       default:
         return HANDLER_ERROR; /* something went wrong */
     }
+  } while (rc == HANDLER_COMEBACK);
+ #ifndef __COVERITY__
+  return HANDLER_ERROR; /* should not happen */ /*(not reached)*/
+ #endif
 }
